@@ -30,7 +30,7 @@ class FileCache(TileCacheBase):
     """
     This class is responsible to store and load the actual tile data.
     """
-    def __init__(self, cache_dir, file_ext, lock_dir=None, directory_layout='tc',
+    def __init__(self, cache_dir, file_ext, directory_layout='tc',
                  link_single_color_images=False, lock_timeout=60.0):
         """
         :param cache_dir: the path where the tile will be stored
